# Providers

<!-- Generated file, do not edit directly. -->



The providers described below are propagated and required by various Swift
build rules. Clients interested in writing custom rules that interface
with the rules in this package should use these providers to communicate
with the Swift build rules as needed.

On this page:

  * [SwiftInfo](#SwiftInfo)
  * [SwiftToolchainInfo](#SwiftToolchainInfo)
  * [SwiftProtoInfo](#SwiftProtoInfo)
  * [SwiftUsageInfo](#SwiftUsageInfo)
<a name="SwiftInfo"></a>
## SwiftInfo

Contains information about the compiled artifacts of a Swift module.

This provider contains a large number of fields and many custom rules may not need to set all of
them. Instead of constructing a `SwiftInfo` provider directly, consider using the
`swift_common.create_swift_info` function, which has reasonable defaults for any fields not
explicitly set.

<table class="fields-table">
  <colgroup>
    <col class="col-field" />
    <col class="col-description" />
  </colgroup>
  <tbody>
    <tr id="SwiftInfo.direct_defines">
      <td><code>direct_defines</code></td>
      <td><p><code>List</code> of <code>string</code>s. The values specified by the <code>defines</code> attribute of the library that directly
propagated this provider.</p></td>
    </tr>
    <tr id="SwiftInfo.direct_swiftdocs">
      <td><code>direct_swiftdocs</code></td>
      <td><p><code>List</code> of <code>File</code>s. The Swift documentation (<code>.swiftdoc</code>) files for the library that directly
propagated this provider.</p></td>
    </tr>
    <tr id="SwiftInfo.direct_swiftmodules">
      <td><code>direct_swiftmodules</code></td>
      <td><p><code>List</code> of <code>File</code>s. The Swift modules (<code>.swiftmodule</code>) for the library that directly propagated this
provider.</p></td>
    </tr>
    <tr id="SwiftInfo.module_name">
      <td><code>module_name</code></td>
      <td><p><code>String</code>. The name of the Swift module represented by the target that directly propagated this
provider.</p>
<p>This field will be equal to the explicitly assigned module name (if present); otherwise, it will be
equal to the autogenerated module name.</p></td>
    </tr>
    <tr id="SwiftInfo.swift_version">
      <td><code>swift_version</code></td>
      <td><p><code>String</code>. The version of the Swift language that was used when compiling the propagating target;
that is, the value passed via the <code>-swift-version</code> compiler flag. This will be <code>None</code> if the flag
was not set.</p></td>
    </tr>
    <tr id="SwiftInfo.transitive_defines">
      <td><code>transitive_defines</code></td>
      <td><p><code>Depset</code> of <code>string</code>s. The transitive <code>defines</code> specified for the library that propagated this
provider and all of its dependencies.</p></td>
    </tr>
    <tr id="SwiftInfo.transitive_modulemaps">
      <td><code>transitive_modulemaps</code></td>
      <td><p><code>Depset</code> of <code>File</code>s. The transitive module map files that will be passed to Clang using the
<code>-fmodule-map-file</code> option.</p></td>
    </tr>
    <tr id="SwiftInfo.transitive_swiftdocs">
      <td><code>transitive_swiftdocs</code></td>
      <td><p><code>Depset</code> of <code>File</code>s. The transitive Swift documentation (<code>.swiftdoc</code>) files emitted by the library
that propagated this provider and all of its dependencies.</p></td>
    </tr>
    <tr id="SwiftInfo.transitive_swiftmodules">
      <td><code>transitive_swiftmodules</code></td>
      <td><p><code>Depset</code> of <code>File</code>s. The transitive Swift modules (<code>.swiftmodule</code>) emitted by the library that
propagated this provider and all of its dependencies.</p></td>
    </tr>
  </tbody>
</table>


<a name="SwiftToolchainInfo"></a>
## SwiftToolchainInfo

Propagates information about a Swift toolchain to compilation and linking rules that use the
toolchain.

<table class="fields-table">
  <colgroup>
    <col class="col-field" />
    <col class="col-description" />
  </colgroup>
  <tbody>
    <tr id="SwiftToolchainInfo.action_environment">
      <td><code>action_environment</code></td>
      <td><p><code>Dict</code>. Environment variables that should be set during any actions spawned to compile or link Swift
code.</p></td>
    </tr>
    <tr id="SwiftToolchainInfo.all_files">
      <td><code>all_files</code></td>
      <td><p>A <code>depset</code> of <code>File</code>s containing all the Swift toolchain files (tools, libraries, and other resource
files) so they can be passed as <code>tools</code> to actions using this toolchain.</p></td>
    </tr>
    <tr id="SwiftToolchainInfo.cc_toolchain_info">
      <td><code>cc_toolchain_info</code></td>
      <td><p>The <code>cc_common.CcToolchainInfo</code> provider from the Bazel C++ toolchain that this Swift toolchain
depends on.</p></td>
    </tr>
    <tr id="SwiftToolchainInfo.clang_executable">
      <td><code>clang_executable</code></td>
      <td><p><code>String</code>. The path to the <code>clang</code> executable, which is used to link binaries.</p></td>
    </tr>
    <tr id="SwiftToolchainInfo.command_line_copts">
      <td><code>command_line_copts</code></td>
      <td><p><code>List</code> of <code>strings</code>. Flags that were passed to Bazel using the <code>--swiftcopt</code> command line flag.
These flags have the highest precedence; they are added to compilation command lines after the
toolchain default flags (<code>SwiftToolchainInfo.swiftc_copts</code>) and after flags specified in the
<code>copts</code> attributes of Swift targets.</p></td>
    </tr>
    <tr id="SwiftToolchainInfo.cpu">
      <td><code>cpu</code></td>
      <td><p><code>String</code>. The CPU architecture that the toolchain is targeting.</p></td>
    </tr>
    <tr id="SwiftToolchainInfo.execution_requirements">
      <td><code>execution_requirements</code></td>
      <td><p><code>Dict</code>. Execution requirements that should be passed to any actions spawned to compile or link
Swift code.</p>
<p>For example, when using an Xcode toolchain, the execution requirements should be such that running
on Darwin is required.</p></td>
    </tr>
    <tr id="SwiftToolchainInfo.implicit_deps">
      <td><code>implicit_deps</code></td>
      <td><p><code>List</code> of <code>Target</code>s. Library targets that should be added as implicit dependencies of any
<code>swift_library</code>, <code>swift_binary</code>, or <code>swift_test</code> target.</p></td>
    </tr>
    <tr id="SwiftToolchainInfo.linker_opts_producer">
      <td><code>linker_opts_producer</code></td>
      <td><p>Skylib <code>partial</code>. A partial function that returns the flags that should be passed to Clang to link a
binary or test target with the Swift runtime libraries.</p>
<p>The partial should be called with two arguments:</p>
<ul>
<li><code>is_static</code>: A <code>Boolean</code> value indicating whether to link against the static or dynamic runtime
libraries.</li>
<li><code>is_test</code>: A <code>Boolean</code> value indicating whether the target being linked is a test target.</li>
</ul></td>
    </tr>
    <tr id="SwiftToolchainInfo.object_format">
      <td><code>object_format</code></td>
      <td><p><code>String</code>. The object file format of the platform that the toolchain is targeting. The currently
supported values are <code>"elf"</code> and <code>"macho"</code>.</p></td>
    </tr>
    <tr id="SwiftToolchainInfo.requested_features">
      <td><code>requested_features</code></td>
      <td><p><code>List</code> of <code>string</code>s. Features that should be implicitly enabled by default for targets built using
this toolchain, unless overridden by the user by listing their negation in the <code>features</code> attribute
of a target/package or in the <code>--features</code> command line flag.</p>
<p>These features determine various compilation and debugging behaviors of the Swift build rules, and
they are also passed to the C++ APIs used when linking (so features defined in CROSSTOOL may be used
here).</p></td>
    </tr>
    <tr id="SwiftToolchainInfo.root_dir">
      <td><code>root_dir</code></td>
      <td><p><code>String</code>. The workspace-relative root directory of the toolchain.</p></td>
    </tr>
    <tr id="SwiftToolchainInfo.stamp">
      <td><code>stamp</code></td>
      <td><p><code>Target</code>. A <code>CcInfo</code>-providing target that should be linked into any binaries that are built with
stamping enabled.</p></td>
    </tr>
    <tr id="SwiftToolchainInfo.supports_objc_interop">
      <td><code>supports_objc_interop</code></td>
      <td><p><code>Boolean</code>. Indicates whether or not the toolchain supports Objective-C interop.</p></td>
    </tr>
    <tr id="SwiftToolchainInfo.swift_worker">
      <td><code>swift_worker</code></td>
      <td><p><code>File</code>. The executable representing the worker executable used to invoke the compiler and other
Swift tools (for both incremental and non-incremental compiles).</p></td>
    </tr>
    <tr id="SwiftToolchainInfo.swiftc_copts">
      <td><code>swiftc_copts</code></td>
      <td><p><code>List</code> of <code>strings</code>. Additional flags that should be passed to <code>swiftc</code> when compiling libraries or
binaries with this toolchain. These flags will come first in compilation command lines, allowing
them to be overridden by <code>copts</code> attributes and <code>--swiftcopt</code> flags.</p></td>
    </tr>
    <tr id="SwiftToolchainInfo.system_name">
      <td><code>system_name</code></td>
      <td><p><code>String</code>. The name of the operating system that the toolchain is targeting.</p></td>
    </tr>
    <tr id="SwiftToolchainInfo.unsupported_features">
      <td><code>unsupported_features</code></td>
      <td><p><code>List</code> of <code>string</code>s. Features that should be implicitly disabled by default for targets built using
this toolchain, unless overridden by the user by listing them in the <code>features</code> attribute of a
target/package or in the <code>--features</code> command line flag.</p>
<p>These features determine various compilation and debugging behaviors of the Swift build rules, and
they are also passed to the C++ APIs used when linking (so features defined in CROSSTOOL may be used
here).</p></td>
    </tr>
  </tbody>
</table>


<a name="SwiftProtoInfo"></a>
## SwiftProtoInfo

Propagates Swift-specific information about a `proto_library`.

<table class="fields-table">
  <colgroup>
    <col class="col-field" />
    <col class="col-description" />
  </colgroup>
  <tbody>
    <tr id="SwiftProtoInfo.module_mappings">
      <td><code>module_mappings</code></td>
      <td><p><code>Sequence</code> of <code>struct</code>s. Each struct contains <code>module_name</code> and <code>proto_file_paths</code> fields that
denote the transitive mappings from <code>.proto</code> files to Swift modules. This allows messages that
reference messages in other libraries to import those modules in generated code.</p></td>
    </tr>
    <tr id="SwiftProtoInfo.pbswift_files">
      <td><code>pbswift_files</code></td>
      <td><p><code>Depset</code> of <code>File</code>s. The transitive Swift source files (<code>.pb.swift</code>) generated from the <code>.proto</code>
files.</p></td>
    </tr>
  </tbody>
</table>


<a name="SwiftUsageInfo"></a>
## SwiftUsageInfo

A provider that indicates that Swift was used by a target or any target that it depends on, and
specifically which toolchain was used.

<table class="fields-table">
  <colgroup>
    <col class="col-field" />
    <col class="col-description" />
  </colgroup>
  <tbody>
    <tr id="SwiftUsageInfo.toolchain">
      <td><code>toolchain</code></td>
      <td><p>The Swift toolchain that was used to build the targets propagating this provider.</p></td>
    </tr>
  </tbody>
</table>

